%% Cooperative Communication with Amplify and Forward Mode
% BPSK Modulation , Flate Rayleigh Fading with WGN &  BER Plots
%% -- Source & Modulator
clc
clear all
close all
ModIndex = 4;
SNRdB = 0:2:20;
NumBits=100e3; % No of Bits To transmit
BPS = log2(ModIndex); % Bits Per Symbol
data=randint(1,NumBits); % data to send ( x )
Txsig=marymod(data,ModIndex); % Modulation 

for i =1:length(SNRdB)

%% --Additive Noise--

N_sd = ( randn( 1,NumBits / BPS ) + 1j * randn( 1,NumBits / BPS ) ) / sqrt( 2 ) *10.^(-SNRdB(i)/20);
N_sr = ( randn( 1,NumBits / BPS ) + 1j * randn( 1,NumBits / BPS ) ) / sqrt( 2 ) *10.^(-SNRdB(i)/20);
N_rd = ( randn( 1,NumBits / BPS ) + 1j * randn( 1,NumBits / BPS ) ) / sqrt( 2 ) *10.^(-SNRdB(i)/20);

% SNR Conversion
SNR=10.^(SNRdB(i)/20);
%%  -- Rayleigh Channels --

chan_sd = 1/sqrt(2)*(  randn(1,  length(Txsig)  ) + j*randn(1,  length(Txsig)  )  ); % Rayleigh channel
chan_sr = 1/sqrt(2)*(  randn(1,  length(Txsig)  ) + j*randn(1,  length(Txsig)  )  ); % Rayleigh channel
chan_rd = 1/sqrt(2)*(  randn(1,  length(Txsig)  ) + j*randn(1,  length(Txsig)  )  ); % Rayleigh channel

%% -- At Relay--
Ysr=sqrt(SNR)*chan_sr.*Txsig * sqrt(10^(-1/10))+ N_sr;% Signal from Source to Relay
% -- Amplification--
beta= sqrt((SNR)./ (sqrt(SNR.*abs(chan_sr).^2)+N_sr)) ; %Amplifying Gain
Y=Ysr.*beta; %Amplified Signal

%% -- At Destination--
Ysd=sqrt(SNR)*chan_sd.*Txsig * sqrt(10^(-1/10))+ N_sd;%Signal from Source to Destination
Yrd=chan_rd.*Y.*sqrt(10^(-1/10))+ N_rd; % Signal from Relay to Destination

%Note: For MRC Assuming that Rx Perfectly Knows Chanel Conditions

% ------- MRC----------
a1=(sqrt(SNR).*conj(chan_sd))./var(N_sd); % MRC weights
a2=(sqrt(SNR).*conj(chan_rd).*conj(chan_sr).*beta)./(((abs(chan_rd).^2).*(abs(beta).^2) +1).*var(N_rd));% MRC weights

MRC=(Ysd.*a1)+(Yrd.*a2); % Combined Signal

% -- Demodulator  --
Rx_sd=marydmod(ModIndex,Ysd./chan_sd,NumBits); % Demodulating Not Cooperated Signal
Rx_Mrc=marydmod(ModIndex,MRC,NumBits);%Demodulating Cooperated Signal
 
%% --Error Comparision
[bitsd,bersd(i)]=biterr(data,Rx_sd);
[bitrd,bermrc(i)]=biterr(data,Rx_Mrc);
end

semilogy(SNRdB,bersd,SNRdB,bermrc)
legend('Without CoOperation','With CoOperation (AF)')
grid on
xlabel('SNR dB')
ylabel('Bit Error Rate')